/*
 *  compare.c - comparing functions
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef COMPARE_H
#define COMPARE_H

#include "osdep.h"
#include "main.h"
#include "debug.h"

int compare_fcn2(const void *arg1, const void *arg2);
int compare_fcn3(const void *arg1, const void *arg2);
/*static inline uint8_t compare_calls(char *call1, char *call2);
static inline uint8_t compare_locs(uint32_t loc1, uint32_t loc2);
static inline uint8_t compare_calls2(char *call1, char *call2);*/
FILE *open_repfile(uint8_t band, int append);
void close_repfile(FILE *fd);
void make_loc_cmp(uint8_t band);
void make_call_cmp(uint8_t band);
int compare_fcn(const void *arg1, const void *arg2);
int compare_claimed_fcn(const void *arg1, const void *arg2);

/* returns 1 if calls is equal with few differenced chars */
static inline uint8_t compare_calls(char *call1, char *call2)
{
	DEBUG_FUNC("compare_calls");

	int len1 = strlen(call1);
	int len2 = strlen(call2);
	int len, i;
	int err = 0;

	/* len = min(len1, len2); */
	len = len1;
	if (len > len2)
		len = len2;

    /* in how much chars calls differs? */
	for (i = 0; i < len; i++)
		if (call1[i] != call2[i])
			err++;

	/* same lengths */
	if (len1 == len2 && len > 4 && err < 3)
		return 1;
	if (len1 == len2 && len > 3 && err < 2)
		return 1;
	/* lengths differs */
	if (abs(len1 - len2) <= 2 && !err)
		return 1;
	if (abs(len1 - len2) < 2 && err < 2)
		return 1;

	return 0; /* too much differences */
}

/* returns 1 if locs differs in max 2 chars */
static inline uint8_t compare_locs(uint32_t loc1, uint32_t loc2)
{
	DEBUG_FUNC("compare_locs");

	char l1[7], l2[7];
	int err = 0, i;

	if (!loc1 || !loc2)
		return 0;
	if (loc1 == loc2)
		return 1;

	num2loc(loc1, l1);
	num2loc(loc2, l2);

	for (i = 0; i < 6; i++)
		if (l1[i] != l2[i])
			err++;

	if (err < 3)
		return 1;

	return 0;
}

/* compare calls without any '/' (strong) after call */
static inline uint8_t compare_calls2(char *call1, char *call2)
{
	DEBUG_FUNC("compare_calls2");

	char call11[SIZE_CALL + 1];
	char call22[SIZE_CALL + 1];
	char *c;

	if (!strcmp(call1, call2))
		return 1; /* calls're same */

	strcpy(call11, call1);
	c = strchr(call11, 0);
	if ((c - 3) < call11)
		return 0; /* too short call */
	/* remove '/' */
	if (*(--c) == '/')
		*c = 0;
	else if (*(--c) == '/')
		*c = 0;
	else if (*(--c) == '/')
		*c = 0;

	strcpy(call22, call2);
	c = strchr(call22, 0);
	if ((c - 3) < call22)
		return 0; /* too short */
	/* remove '/' */
	if (*(--c) == '/')
		*c = 0;
	else if (*(--c) == '/')
		*c = 0;
	else if (*(--c) == '/')
		*c = 0;

	/* compare again */
	if (!strcmp(call11, call22))
		return 1;

	return 0;
}


#endif /* COMPARE_H */
